<?php

declare(strict_types=1);

namespace Drupal\konfhub\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\konfhub\TicketDetailsManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\Serializer\Encoder\JsonEncoder;
use Symfony\Component\Serializer\Serializer;

/**
 * Returns responses for Konfhub routes.
 */
final class KonfhubController extends ControllerBase {

  /**
   * The controller constructor.
   */
  public function __construct(
    protected Serializer $serializer,
    protected array $serializerFormats,
    protected TicketDetailsManager $ticketDetailsManager,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    if ($container->hasParameter('serializer.formats') && $container->has('serializer')) {
      $serializer = $container->get('serializer');
      $formats = $container->getParameter('serializer.formats');
    }
    else {
      $formats = ['json'];
      $encoders = [new JsonEncoder()];
      $serializer = new Serializer([], $encoders);
    }
    return new self(
      $serializer,
      $formats,
      $container->get('konfhub.ticket_details_manager'),
    );
  }

  /**
   * Listens to the webhook requests from the Konfhub.
   */
  public function webhookListener(Request $request): JsonResponse {
    $format = $request->getRequestFormat();

    if (!in_array($format, $this->serializerFormats)) {
      throw new BadRequestHttpException("Unrecognized format: $format.");
    }

    $content = $request->getContent();
    $payload = $this->serializer->decode($content, 'json');

    $status = $this->ticketDetailsManager->processData($payload);

    return new JsonResponse(['status' => $status]);
  }

}
